# -*- coding: utf-8 -*-
u"""
Created on 2017-6-7

@author: cheng.li
"""


class RootFinder(object):

    def putY(self, y):
        pass

    def nextX(self):
        pass

    def bracketWidth(self):
        pass
