# -*- coding: utf-8 -*-
u"""
Created on 2017-4-3

@author: cheng.li
"""


cdef double DBL_MIN
cdef double QL_EPSILON
cdef double M_SQRT2
cdef double M_SQRT_2
cdef double M_SQRTPI
cdef double M_1_SQRTPI
cdef double M_PI
cdef double NAN