# -*- coding: utf-8 -*-
u"""
Created on 2017-1-30

@author: cheng.li
"""

cdef public enum Weekdays:
    Sunday = 1
    Monday = 2
    Tuesday = 3
    Wednesday = 4
    Thursday = 5
    Friday = 6
    Saturday = 7